@@ -30,6 +30,7 @@ except:
 import gconf
 import gobject
 import gtk
+import json
 import logging
 import logging.config
 import os
@@ -45,8 +46,8 @@ from urlparse import urlsplit
 from xdg.DesktopEntry import DesktopEntry
 from xml.sax.saxutils import escape as xml_escape
 
-import guake.globalhotkeys
 import guake.notifier
+import keybinder
 
 from guake.about import AboutDialog
 from guake.common import _
@@ -108,6 +109,9 @@ GDK_WINDOW_STATE_ICONIFIED = 2
 GDK_WINDOW_STATE_STICKY = 8
 GDK_WINDOW_STATE_ABOVE = 32
 
+# Transparency max level (should be always 100)
+MAX_TRANSPARENCY = 100
+
 
 log = logging.getLogger(__name__)
 
@@ -157,9 +161,6 @@ class Guake(SimpleGladeApp):
         self.debug_mode = self.client.get_bool(KEY('/general/debug_mode'))
         self.setupLogging()
 
-        # setting global hotkey and showing a pretty notification =)
-        guake.globalhotkeys.init()
-
         # Cannot use "getattr(gtk.Window().get_style(), "base")[int(gtk.STATE_SELECTED)]"
         # since theme has not been applied before first show_all
         self.selected_color = None
@@ -169,11 +170,12 @@ class Guake(SimpleGladeApp):
         self.forceHide = False
         self.preventHide = False
 
-        # trayicon!
+        # trayicon! Using SVG handles better different OS trays
+        img = pixmapfile('guake-tray.svg')
+
         try:
             import appindicator
         except ImportError:
-            img = pixmapfile('guake-tray.png')
             self.tray_icon = gtk.status_icon_new_from_file(img)
             self.tray_icon.set_tooltip(_('Guake Terminal'))
             self.tray_icon.connect('popup-menu', self.show_menu)
@@ -181,7 +183,7 @@ class Guake(SimpleGladeApp):
         else:
             self.tray_icon = appindicator.Indicator(
                 _("guake-indicator"), _("guake-tray"), appindicator.CATEGORY_OTHER)
-            self.tray_icon.set_icon("guake-tray")
+            self.tray_icon.set_icon(img)
             self.tray_icon.set_status(appindicator.STATUS_ACTIVE)
             menu = self.get_widget('tray-menu')
             show = gtk.MenuItem(_('Show'))
@@ -191,6 +193,9 @@ class Guake(SimpleGladeApp):
             menu.prepend(show)
             self.tray_icon.set_menu(menu)
 
+        ipath = pixmapfile('add_tab.png')
+        self.get_widget('image2').set_from_file(ipath)
+
         # important widgets
         self.window = self.get_widget('window-root')
         self.mainframe = self.get_widget('mainframe')
@@ -247,6 +252,16 @@ class Guake(SimpleGladeApp):
         # holds the timestamp of the previous show/hide action
         self.prev_showhide_time = 0
 
+        # holds transparency level
+        self.transparency = 0
+
+        # Controls the transparency state needed for function accel_toggle_transparency
+        self.transparency_toggled = False
+
+        # load cumstom command menu and menuitems from config file
+        self.custom_command_menuitem = None
+        self.load_custom_commands()
+
         # double click stuff
         def double_click(hbox, event):
             """Handles double clicks on tabs area and when receive
@@ -257,6 +272,25 @@ class Guake(SimpleGladeApp):
         evtbox = self.get_widget('event-tabs')
         evtbox.connect('button-press-event', double_click)
 
+        def scroll_manager(hbox, event):
+            adj = self.get_widget('tabs-scrolledwindow').get_hadjustment()
+            adj.set_page_increment(1)
+            if event.direction == gtk.gdk.SCROLL_DOWN:
+                if self.notebook.get_current_page() + 1 < self.notebook.get_tab_count():
+                    self.notebook.next_page()
+                else:
+                    return
+
+            if event.direction == gtk.gdk.SCROLL_UP:
+                self.notebook.prev_page()
+
+            current_page = self.notebook.get_current_page()
+            tab = self.tabs.get_children()[current_page]
+            rectangle = tab.get_allocation()
+            adj.set_value(rectangle.x)
+
+        evtbox.connect('scroll-event', scroll_manager)
+
         # Flag to prevent guake hide when window_losefocus is true and
         # user tries to use the context menu.
         self.showing_context_menu = False
@@ -283,15 +317,13 @@ class Guake(SimpleGladeApp):
         self.window.connect('delete-event', destroy)
         self.window.connect('window-state-event', window_event)
 
-        # Flag to completely disable losefocus hiding
-        self.disable_losefocus_hiding = False
-
         # this line is important to resize the main window and make it
         # smaller.
         self.window.set_geometry_hints(min_width=1, min_height=1)
 
         # special trick to avoid the "lost guake on Ubuntu 'Show Desktop'" problem.
-        # DOCK makes the window foundable after having being "lost" after "Show Desktop"
+        # DOCK makes the window foundable after having being "lost" after "Show
+        # Desktop"
         self.window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
         # Restore back to normal behavior
         self.window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_NORMAL)
@@ -304,8 +336,8 @@ class Guake(SimpleGladeApp):
 
         # loading and setting up configuration stuff
         GConfHandler(self)
+        self.hotkeys = keybinder
         GConfKeyHandler(self)
-        self.hotkeys = guake.globalhotkeys.GlobalHotkey()
         self.load_config()
 
         key = self.client.get_string(GKEY('show_hide'))
@@ -318,15 +350,7 @@ class Guake(SimpleGladeApp):
         if self.client.get_bool(KEY('/general/start_fullscreen')):
             self.fullscreen()
 
-        if not self.hotkeys.bind(key, self.show_hide):
-            guake.notifier.show_message(
-                _('Guake Terminal'),
-                _('A problem happened when binding <b>%s</b> key.\n'
-                  'Please use Guake Preferences dialog to choose another '
-                  'key') % xml_escape(label), filename)
-            self.client.set_bool(KEY('/general/use_trayicon'), True)
-
-        elif self.client.get_bool(KEY('/general/use_popup')):
+        if self.client.get_bool(KEY('/general/use_popup')):
             # Pop-up that shows that guake is working properly (if not
             # unset in the preferences windows)
             guake.notifier.show_message(
@@ -334,6 +358,78 @@ class Guake(SimpleGladeApp):
                 _('Guake is now running,\n'
                   'press <b>%s</b> to use it.') % xml_escape(label), filename)
 
+    # load the custom commands infrastucture
+    def load_custom_commands(self):
+        if hasattr(self, 'custom_command_menuitem') and self.custom_command_menuitem:
+            self.get_widget(
+                'context-menu').remove(self.custom_command_menuitem)
+        menu = gtk.MenuItem("Custom Commands")
+        custom_commands_menu = gtk.Menu()
+        self.get_custom_commands(custom_commands_menu)
+        menu.set_submenu(custom_commands_menu)
+        menu.show()
+        self.get_widget('context-menu').insert(menu,
+                                               self.context_menu_get_insert_pos())
+        self.custom_command_menuitem = menu
+
+    # returns position where the custom cmd must be placed on the context-menu
+    def context_menu_get_insert_pos(self):
+        # assuming that the quit menuitem is always the last one and with a
+        # separator before him
+        return len(self.get_widget('context-menu').get_children()) - 2
+
+    # function to read commands stored at /general/custom_command_file and
+    # launch the context menu builder
+    def get_custom_commands(self, menu):
+        custom_command_file_path = self.client.get_string(KEY('/general/custom_command_file'))
+        if not custom_command_file_path:
+            return
+        file_name = os.path.expanduser(custom_command_file_path)
+        if not file_name:
+            return
+        try:
+            with open(file_name) as f:
+                data_file = f.read()
+        except:
+            data_file = None
+        if not data_file:
+            return
+
+        try:
+            custom_commands = json.loads(data_file)
+            for json_object in custom_commands:
+                self.parse_custom_commands(json_object, menu)
+
+        except Exception:
+            log.exception(
+                "Invalid custom command file %s. Exception:", data_file)
+
+    # function to build the custom commands menu and menuitems
+    def parse_custom_commands(self, json_object, menu):
+        if json_object.get('type') == "menu":
+            newmenu = gtk.Menu()
+            newmenuitem = gtk.MenuItem(json_object['description'])
+            newmenuitem.set_submenu(newmenu)
+            newmenuitem.show()
+            menu.append(newmenuitem)
+            for item in json_object['items']:
+                self.parse_custom_commands(item, newmenu)
+        else:
+            menu_item = gtk.MenuItem(json_object['description'])
+            custom_command = ""
+            space = ""
+            for command in json_object['cmd']:
+                custom_command += (space + command)
+                space = " "
+
+            menu_item.connect("activate", self.execute_context_menu_cmd, custom_command)
+            menu.append(menu_item)
+            menu_item.show()
+
+    # execute contextual menu call
+    def execute_context_menu_cmd(self, item, cmd):
+        self.execute_command(cmd)
+
     def setupLogging(self):
         if self.debug_mode:
             base_logging_level = logging.DEBUG
@@ -382,7 +478,7 @@ class Guake(SimpleGladeApp):
         log.debug(text)
 
     def printInfo(self, text):
-        log.debug(text)
+        log.info(text)
 
     def set_background_transparency(self, transparency):
         for t in self.notebook.iter_terminals():
@@ -462,13 +558,14 @@ class Guake(SimpleGladeApp):
 
         window_rect = self.window.get_size()
         self.window.resize(window_rect[0], y)
-        # self.client.set_int(KEY('/general/window_height'), int(percent))
+        self.client.set_int(KEY('/general/window_height'), int(percent))
+        self.client.set_float(KEY('/general/window_height_f'), float(percent))
 
     def on_window_losefocus(self, window, event):
         """Hides terminal main window when it loses the focus and if
         the window_losefocus gconf variable is True.
         """
-        if self.disable_losefocus_hiding or self.showing_context_menu:
+        if self.showing_context_menu:
             return
 
         if self.prompt_dialog is not None:
@@ -480,8 +577,8 @@ class Guake(SimpleGladeApp):
         value = self.client.get_bool(KEY('/general/window_losefocus'))
         visible = window.get_property('visible')
         if value and visible:
-            self.losefocus_time = \
-                gtk.gdk.x11_get_server_time(self.window.window)
+            self.losefocus_time = gtk.gdk.x11_get_server_time(
+                self.window.window)
             self.hide()
 
     def show_menu(self, status_icon, button, activate_time):
@@ -520,8 +617,8 @@ class Guake(SimpleGladeApp):
 
         self.get_widget('separator_search').set_visible(False)
         if current_selection:
-            self.get_widget('context_search_on_web').set_label(
-                _("Search on Web: '%s'") % current_selection)
+            self.get_widget('context_search_on_web').set_label(_("Search on Web: '%s'") %
+                                                               current_selection)
             self.get_widget('context_search_on_web').set_visible(True)
             self.get_widget('separator_search').set_visible(True)
         else:
@@ -584,7 +681,7 @@ class Guake(SimpleGladeApp):
 
     def window_event(self, window, event):
         state = event.new_window_state
-        log.debug("Received window state event. Window: %s. Event: %s", window, state)
+        log.debug("Received window state event: %s", state)
 
     def show_hide(self, *args):
         """Toggles the main window visibility
@@ -610,6 +707,8 @@ class Guake(SimpleGladeApp):
             return
         self.prev_showhide_time = event_time
 
+        log.debug("")
+        log.debug("=" * 80)
         log.debug("Window display")
         if self.window.window:
             cur_state = int(self.window.window.get_state())
@@ -697,11 +796,12 @@ class Guake(SimpleGladeApp):
             time = 0
 
         # When minized, the window manager seems to refuse to resume
-        log.debug("self.window: %s. Dir=%s", type(self.window), dir(self.window))
+        # log.debug("self.window: %s. Dir=%s", type(self.window), dir(self.window))
         # is_iconified = self.is_iconified()
         # if is_iconified:
-        #     log.debug("Is iconified. Ubuntu Trick => removing skip_taskbar_hint and skip_pager_hint "
-        #           "so deiconify can work!")
+        #     log.debug("Is iconified. Ubuntu Trick => "
+        #               "removing skip_taskbar_hint and skip_pager_hint "
+        #               "so deiconify can work!")
         #     self.get_widget('window-root').set_skip_taskbar_hint(False)
         #     self.get_widget('window-root').set_skip_pager_hint(False)
         #     self.get_widget('window-root').set_urgency_hint(False)
@@ -712,6 +812,9 @@ class Guake(SimpleGladeApp):
         #     log.debug("get_urgency_hint: {}".format(
         #         self.get_widget('window-root').get_urgency_hint()))
         #     glib.timeout_add_seconds(1, lambda: self.timeout_restore(time))
+        #
+        log.debug("Current window geometry: %r",
+                  self.window.window.get_geometry())
 
         log.debug("order to present and deiconify")
         self.window.present()
@@ -728,7 +831,8 @@ class Guake(SimpleGladeApp):
         #     self.get_widget('window-root').set_skip_pager_hint(False)
         #     self.get_widget('window-root').set_urgency_hint(False)
 
-        # This is here because vte color configuration works only after the widget is shown.
+        # This is here because vte color configuration works only after the
+        # widget is shown.
         self.client.notify(KEY('/style/font/color'))
         self.client.notify(KEY('/style/background/color'))
 
@@ -794,12 +898,10 @@ class Guake(SimpleGladeApp):
 
         # http://askubuntu.com/questions/70296/is-there-an-environment-variable-that-is-set-for-unity
         if float(linux_distrib[1]) - 0.01 < 11.10:
-            if os.environ.get('DESKTOP_SESSION').lower() == "gnome".lower():
-                log.debug("Unity detected")
+            if os.environ.get('DESKTOP_SESSION', '').lower() == "gnome".lower():
                 return True
         else:
-            if os.environ.get('XDG_CURRENT_DESKTOP').lower() == "unity".lower():
-                log.debug("Unity detected")
+            if os.environ.get('XDG_CURRENT_DESKTOP', '').lower() == "unity".lower():
                 return True
         return False
 
@@ -820,15 +922,18 @@ class Guake(SimpleGladeApp):
         halignment = self.client.get_int(KEY('/general/window_halignment'))
         valignment = self.client.get_int(KEY('/general/window_valignment'))
 
-        # log.debug("height_percents", height_percents)
-        # log.debug("width_percents", width_percents)
-        # log.debug("halignment", halignment)
-        # log.debug("valignment", valignment)
+        log.debug("set_final_window_rect")
+        log.debug("height_percents = %s", height_percents)
+        log.debug("width_percents = %s", width_percents)
+        log.debug("halignment = %s", halignment)
+        log.debug("valignment = %s", valignment)
 
         # get the rectangle just from the destination monitor
         screen = self.window.get_screen()
         monitor = self.get_final_window_monitor()
         window_rect = screen.get_monitor_geometry(monitor)
+        log.debug("Current screen geometry: %r", window_rect)
+        log.debug("is unity: %s", self.is_using_unity())
 
         if self.is_using_unity():
 
@@ -879,10 +984,11 @@ class Guake(SimpleGladeApp):
         window_rect.height = window_rect.height * height_percents / 100
         window_rect.width = window_rect.width * width_percents / 100
 
-        # log.debug("window_rect.x", window_rect.x)
-        # log.debug("window_rect.y", window_rect.y)
-        # log.debug("window_rect.height", window_rect.height)
-        # log.debug("window_rect.width", window_rect.width)
+        log.debug("BEFORE SETTINGS APPLICATION")
+        log.debug("window_rect.x: %s", window_rect.x)
+        log.debug("window_rect.y: %s", window_rect.y)
+        log.debug("window_rect.height: %s", window_rect.height)
+        log.debug("window_rect.width: %s", window_rect.width)
 
         if window_rect.width < total_width:
             if halignment == ALIGN_CENTER:
@@ -898,9 +1004,14 @@ class Guake(SimpleGladeApp):
             if valignment == ALIGN_BOTTOM:
                 window_rect.y += (total_height - window_rect.height)
 
+        log.debug("RESIZING MAIN WINDOW TO THE FOLLOWING VALUES:")
+        log.debug("window_rect.x: %s", window_rect.x)
+        log.debug("window_rect.y: %s", window_rect.y)
+        log.debug("window_rect.height: %s", window_rect.height)
+        log.debug("window_rect.width: %s", window_rect.width)
+
         self.window.resize(window_rect.width, window_rect.height)
         self.window.move(window_rect.x, window_rect.y)
-        # log.debug("Moving/Resizing to: window_rect", window_rect)
 
         return window_rect
 
@@ -923,6 +1034,7 @@ class Guake(SimpleGladeApp):
         self.client.notify(KEY('/general/history_size'))
         self.client.notify(KEY('/general/show_resizer'))
         self.client.notify(KEY('/general/use_vte_titles'))
+        self.client.notify(KEY('/general/max_tab_name_length'))
         self.client.notify(KEY('/general/quick_open_enable'))
         self.client.notify(KEY('/general/quick_open_command_line'))
         self.client.notify(KEY('/style/cursor_shape'))
@@ -930,6 +1042,7 @@ class Guake(SimpleGladeApp):
         self.client.notify(KEY('/style/font/color'))
         self.client.notify(KEY('/style/font/palette'))
         self.client.notify(KEY('/style/font/palette_name'))
+        self.client.notify(KEY('/style/font/allow_bold'))
         self.client.notify(KEY('/style/background/color'))
         self.client.notify(KEY('/style/background/image'))
         self.client.notify(KEY('/style/background/transparency'))
@@ -967,6 +1080,11 @@ class Guake(SimpleGladeApp):
         else:
             gtk.main_quit()
 
+    def accel_reset_terminal(self, *args):
+        """Callback to reset and clean the terminal"""
+        self.reset_terminal()
+        return True
+
     def accel_zoom_in(self, *args):
         """Callback to zoom in.
         """
@@ -1007,6 +1125,8 @@ class Guake(SimpleGladeApp):
         """Callback to increase transparency.
         """
         transparency = self.client.get_int(KEY('/style/background/transparency'))
+        if transparency >= MAX_TRANSPARENCY:
+            return True
         self.client.set_int(KEY('/style/background/transparency'), int(transparency) + 2)
         return True
 
@@ -1014,9 +1134,23 @@ class Guake(SimpleGladeApp):
         """Callback to decrease transparency.
         """
         transparency = self.client.get_int(KEY('/style/background/transparency'))
+        if transparency <= 0:
+            return True
         self.client.set_int(KEY('/style/background/transparency'), int(transparency) - 2)
         return True
 
+    def accel_toggle_transparency(self, *args):
+        """Callback to toggle transparency.
+        """
+        if self.transparency_toggled:
+            self.client.set_int(KEY('/style/background/transparency'), int(self.transparency))
+            self.transparency_toggled = False
+            return True
+        self.transparency = self.client.get_int(KEY('/style/background/transparency'))
+        self.client.set_int(KEY('/style/background/transparency'), MAX_TRANSPARENCY)
+        self.transparency_toggled = True
+        return True
+
     def accel_add(self, *args):
         """Callback to add a new tab. Called by the accel key.
         """
@@ -1099,6 +1233,7 @@ class Guake(SimpleGladeApp):
         """Callback toggle the fullscreen status of the main
         window. Called by the accel key.
         """
+
         if not self.is_fullscreen:
             self.fullscreen()
         else:
@@ -1109,8 +1244,11 @@ class Guake(SimpleGladeApp):
         """Callback toggle whether the window should hide when it loses
         focus. Called by the accel key.
         """
-        # use temporary setting -- don't change conf key
-        self.disable_losefocus_hiding = not self.disable_losefocus_hiding
+
+        if self.client.get_bool(KEY('/general/window_losefocus')):
+            self.client.set_bool(KEY('/general/window_losefocus'), False)
+        else:
+            self.client.set_bool(KEY('/general/window_losefocus'), True)
         return True
 
     def fullscreen(self):
@@ -1125,6 +1263,10 @@ class Guake(SimpleGladeApp):
             self.toolbar.hide()
 
     def unfullscreen(self):
+
+        # Fixes "Guake cannot restore from fullscreen" (#628)
+        self.window.unmaximize()
+
         self.set_final_window_rect()
         self.window.unfullscreen()
         self.is_fullscreen = False
@@ -1158,7 +1300,11 @@ class Guake(SimpleGladeApp):
         tab = self.tabs.get_children()[page]
         # if tab has been renamed by user, don't override.
         if not getattr(tab, 'custom_label_set', False):
-            tab.set_label(vte.get_window_title())
+            vte_title = vte.get_window_title()
+            max_name_length = self.client.get_int(KEY("/general/max_tab_name_length"))
+            if len(vte_title) > max_name_length and max_name_length is not 0:
+                vte_title = vte_title[:max_name_length]
+            tab.set_label(vte_title)
 
     def on_rename_current_tab_activate(self, *args):
         """Shows a dialog to rename the current tab.
@@ -1187,16 +1333,24 @@ class Guake(SimpleGladeApp):
         entry.reparent(vbox)
 
         # don't hide on lose focus until the rename is finished
-        current_hide_setting = self.disable_losefocus_hiding
-        self.disable_losefocus_hiding = True
+        self.preventHide = True
         response = dialog.run()
-        self.disable_losefocus_hiding = current_hide_setting
+        self.preventHide = False
 
         if response == gtk.RESPONSE_ACCEPT:
+            max_name_length = self.client.get_int(KEY("/general/max_tab_name_length"))
             new_text = entry.get_text()
+            if len(new_text) > max_name_length and max_name_length is not 0:
+                new_text = new_text[:max_name_length]
+
             self.selected_tab.set_label(new_text)
             # if user sets empty name, consider he wants default behavior.
             setattr(self.selected_tab, 'custom_label_set', bool(new_text))
+
+            # holds custom label name of the tab,
+            # we need this to restore the name if the max length is changed
+            setattr(self.selected_tab, 'custom_label_text', new_text)
+
             # trigger titling handler in case that custom label has been reset
             current_vte = self.notebook.get_current_terminal()
             current_vte.emit('window-title-changed')
@@ -1297,9 +1451,11 @@ class Guake(SimpleGladeApp):
         active_terminal = self.notebook.get_current_terminal()
         directory = os.path.expanduser('~')
         if active_terminal:
-            cwd = os.readlink("/proc/%d/cwd" % active_terminal.get_pid())
-            if os.path.exists(cwd):
-                directory = cwd
+            active_pid = active_terminal.get_pid()
+            if active_pid:
+                cwd = os.readlink("/proc/{0}/cwd".format(active_pid))
+                if os.path.exists(cwd):
+                    directory = cwd
         return directory
 
     def get_fork_params(self, default_params=None):
@@ -1405,25 +1561,25 @@ class Guake(SimpleGladeApp):
         pid = box.terminal.fork_command(**final_params)
         if libutempter is not None:
             # After the fork_command we add this new tty to utmp !
-            libutempter.utempter_add_record(box.terminal.get_pty(), os.uname()[1])
+            libutempter.utempter_add_record(
+                box.terminal.get_pty(), os.uname()[1])
         box.terminal.pid = pid
 
         # Adding a new radio button to the tabbar
         label = box.terminal.get_window_title() or _("Terminal")
         tabs = self.tabs.get_children()
         parent = tabs and tabs[0] or None
-
         bnt = gtk.RadioButton(group=parent, label=label, use_underline=False)
         bnt.set_property('can-focus', False)
         bnt.set_property('draw-indicator', False)
         bnt.connect('button-press-event', self.show_tab_menu)
         bnt.connect('button-press-event', self.show_rename_current_tab_dialog)
-        bnt.connect('clicked',
-                    lambda *x: self.notebook.set_current_page(
-                        self.notebook.page_num(box)
-                    ))
+        bnt.connect('clicked', lambda *x: self.notebook.set_current_page(
+            self.notebook.page_num(box)
+        ))
         if self.selected_color is not None:
-            bnt.modify_bg(gtk.STATE_ACTIVE, gtk.gdk.Color(str(self.selected_color)))
+            bnt.modify_bg(gtk.STATE_ACTIVE, gtk.gdk.Color(
+                str(self.selected_color)))
         drag_drop_type = ("text/plain", gtk.TARGET_SAME_APP, 80)
         bnt.drag_dest_set(gtk.DEST_DEFAULT_ALL, [drag_drop_type], gtk.gdk.ACTION_MOVE)
         bnt.connect("drag_data_received", self.on_drop_tab)
@@ -1448,7 +1604,10 @@ class Guake(SimpleGladeApp):
         current_term.copy_clipboard()
         current_term.select_none()
         guake_clipboard = gtk.clipboard_get()
-        current_selection = guake_clipboard.wait_for_text().rstrip()
+        current_selection = guake_clipboard.wait_for_text()
+        if not current_selection:
+            return
+        current_selection = current_selection.rstrip()
 
         dialog = gtk.FileChooserDialog(_("Save to..."),
                                        self.window,
@@ -1530,7 +1689,8 @@ class Guake(SimpleGladeApp):
         self.move_tab(old_tab_pos, new_tab_pos)
 
     def move_tab(self, old_tab_pos, new_tab_pos):
-        self.notebook.reorder_child(self.notebook.get_nth_page(old_tab_pos), new_tab_pos)
+        self.notebook.reorder_child(
+            self.notebook.get_nth_page(old_tab_pos), new_tab_pos)
         self.tabs.reorder_child(self.tabs.get_children()[old_tab_pos], new_tab_pos)
         self.notebook.set_current_page(new_tab_pos)
 
@@ -1622,4 +1782,10 @@ class Guake(SimpleGladeApp):
             self.mainframe.reorder_child(self.notebook, 2)
         else:
             self.mainframe.reorder_child(self.notebook, 0)
-        self.mainframe.pack_start(self.mainframe, expand=True, fill=True, padding=0)
+        self.mainframe.pack_start(self.notebook, expand=True, fill=True, padding=0)
+
+    def reset_terminal(self, directory=None):
+        self.preventHide = True
+        current_term = self.notebook.get_current_terminal()
+        current_term.reset(full=True, clear_history=True)
+        self.preventHide = False
